# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.4
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import re  # noqa: F401
import io
import warnings

from pydantic import validate_arguments, ValidationError
from typing_extensions import Annotated

from typing import List, Optional

from stadiamaps.models.isochrone_request import IsochroneRequest
from stadiamaps.models.isochrone_response import IsochroneResponse
from stadiamaps.models.locate_object import LocateObject
from stadiamaps.models.map_match_request import MapMatchRequest
from stadiamaps.models.map_match_route_response import MapMatchRouteResponse
from stadiamaps.models.matrix_request import MatrixRequest
from stadiamaps.models.matrix_response import MatrixResponse
from stadiamaps.models.nearest_roads_request import NearestRoadsRequest
from stadiamaps.models.optimized_route_request import OptimizedRouteRequest
from stadiamaps.models.route_request import RouteRequest
from stadiamaps.models.route_response import RouteResponse
from stadiamaps.models.trace_attributes_request import TraceAttributesRequest
from stadiamaps.models.trace_attributes_response import TraceAttributesResponse

from stadiamaps.api_client import ApiClient
from stadiamaps.api_response import ApiResponse
from stadiamaps.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class RoutingApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_arguments
    def isochrone(self, isochrone_request : Optional[IsochroneRequest] = None, **kwargs) -> IsochroneResponse:  # noqa: E501
        """Calculate areas of equal travel time from a location.  # noqa: E501

        The isochrone API lets you compute or visualize areas of roughly equal travel time based on the routing graph. The resulting polygon can be rendered on a map and shaded much like elevation contours and used for exploring urban mobility.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.isochrone(isochrone_request, async_req=True)
        >>> result = thread.get()

        :param isochrone_request:
        :type isochrone_request: IsochroneRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: IsochroneResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the isochrone_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.isochrone_with_http_info(isochrone_request, **kwargs)  # noqa: E501

    @validate_arguments
    def isochrone_with_http_info(self, isochrone_request : Optional[IsochroneRequest] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Calculate areas of equal travel time from a location.  # noqa: E501

        The isochrone API lets you compute or visualize areas of roughly equal travel time based on the routing graph. The resulting polygon can be rendered on a map and shaded much like elevation contours and used for exploring urban mobility.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.isochrone_with_http_info(isochrone_request, async_req=True)
        >>> result = thread.get()

        :param isochrone_request:
        :type isochrone_request: IsochroneRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(IsochroneResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'isochrone_request'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method isochrone" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['isochrone_request'] is not None:
            _body_params = _params['isochrone_request']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['ApiKeyAuth']  # noqa: E501

        _response_types_map = {
            '200': "IsochroneResponse",
        }

        return self.api_client.call_api(
            '/isochrone/v1', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def map_match(self, map_match_request : Optional[MapMatchRequest] = None, **kwargs) -> MapMatchRouteResponse:  # noqa: E501
        """Match a recorded route to the road network.  # noqa: E501

        The map matching API transforms a recorded route into navigation instructions like you would get from the `route` endpoint. The input can be in the form of either an encoded polyline, or (optionally) timestamped coordinates.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.map_match(map_match_request, async_req=True)
        >>> result = thread.get()

        :param map_match_request:
        :type map_match_request: MapMatchRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: MapMatchRouteResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the map_match_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.map_match_with_http_info(map_match_request, **kwargs)  # noqa: E501

    @validate_arguments
    def map_match_with_http_info(self, map_match_request : Optional[MapMatchRequest] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Match a recorded route to the road network.  # noqa: E501

        The map matching API transforms a recorded route into navigation instructions like you would get from the `route` endpoint. The input can be in the form of either an encoded polyline, or (optionally) timestamped coordinates.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.map_match_with_http_info(map_match_request, async_req=True)
        >>> result = thread.get()

        :param map_match_request:
        :type map_match_request: MapMatchRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(MapMatchRouteResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'map_match_request'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method map_match" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['map_match_request'] is not None:
            _body_params = _params['map_match_request']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['ApiKeyAuth']  # noqa: E501

        _response_types_map = {
            '200': "MapMatchRouteResponse",
            '400': None,
            '500': None,
        }

        return self.api_client.call_api(
            '/map_match/v1', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def nearest_roads(self, nearest_roads_request : Optional[NearestRoadsRequest] = None, **kwargs) -> List[LocateObject]:  # noqa: E501
        """Find the nearest roads to the set of input locations.  # noqa: E501

        The nearest roads API allows you query for detailed information about streets and intersections near the input locations.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.nearest_roads(nearest_roads_request, async_req=True)
        >>> result = thread.get()

        :param nearest_roads_request:
        :type nearest_roads_request: NearestRoadsRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: List[LocateObject]
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the nearest_roads_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.nearest_roads_with_http_info(nearest_roads_request, **kwargs)  # noqa: E501

    @validate_arguments
    def nearest_roads_with_http_info(self, nearest_roads_request : Optional[NearestRoadsRequest] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Find the nearest roads to the set of input locations.  # noqa: E501

        The nearest roads API allows you query for detailed information about streets and intersections near the input locations.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.nearest_roads_with_http_info(nearest_roads_request, async_req=True)
        >>> result = thread.get()

        :param nearest_roads_request:
        :type nearest_roads_request: NearestRoadsRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(List[LocateObject], status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'nearest_roads_request'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method nearest_roads" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['nearest_roads_request'] is not None:
            _body_params = _params['nearest_roads_request']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['ApiKeyAuth']  # noqa: E501

        _response_types_map = {
            '200': "List[LocateObject]",
        }

        return self.api_client.call_api(
            '/nearest_roads/v1', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def optimized_route(self, optimized_route_request : Optional[OptimizedRouteRequest] = None, **kwargs) -> RouteResponse:  # noqa: E501
        """Calculate an optimized route between a known start and end point.  # noqa: E501

        The optimized route API is a mix of the matrix and normal route API. For an optimized route, the start and end point are fixed, but the intermediate points will be re-ordered to form an optimal route visiting all nodes once. Note that all matrix endpoints have a limit of 1000 elements, regardless of the costing/mode of travel. A matrix request with 3 inputs and 5 outputs has 3 x 5 = 15 elements. This means you could send a 100 x 10 or 20 x 50 matrix request (each having 1000 elements), but not 40 x 30 as it has 1200 elements.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.optimized_route(optimized_route_request, async_req=True)
        >>> result = thread.get()

        :param optimized_route_request:
        :type optimized_route_request: OptimizedRouteRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: RouteResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the optimized_route_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.optimized_route_with_http_info(optimized_route_request, **kwargs)  # noqa: E501

    @validate_arguments
    def optimized_route_with_http_info(self, optimized_route_request : Optional[OptimizedRouteRequest] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Calculate an optimized route between a known start and end point.  # noqa: E501

        The optimized route API is a mix of the matrix and normal route API. For an optimized route, the start and end point are fixed, but the intermediate points will be re-ordered to form an optimal route visiting all nodes once. Note that all matrix endpoints have a limit of 1000 elements, regardless of the costing/mode of travel. A matrix request with 3 inputs and 5 outputs has 3 x 5 = 15 elements. This means you could send a 100 x 10 or 20 x 50 matrix request (each having 1000 elements), but not 40 x 30 as it has 1200 elements.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.optimized_route_with_http_info(optimized_route_request, async_req=True)
        >>> result = thread.get()

        :param optimized_route_request:
        :type optimized_route_request: OptimizedRouteRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(RouteResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'optimized_route_request'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method optimized_route" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['optimized_route_request'] is not None:
            _body_params = _params['optimized_route_request']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['ApiKeyAuth']  # noqa: E501

        _response_types_map = {
            '200': "RouteResponse",
            '400': None,
            '500': None,
        }

        return self.api_client.call_api(
            '/optimized_route/v1', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def route(self, route_request : Optional[RouteRequest] = None, **kwargs) -> RouteResponse:  # noqa: E501
        """Get turn by turn routing instructions between two or more locations.  # noqa: E501

        The route (turn-by-turn) API computes routes between two or more locations. It supports a variety of tunable costing methods, and supports routing through intermediate waypoints and discontinuous multi-leg routes.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.route(route_request, async_req=True)
        >>> result = thread.get()

        :param route_request:
        :type route_request: RouteRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: RouteResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the route_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.route_with_http_info(route_request, **kwargs)  # noqa: E501

    @validate_arguments
    def route_with_http_info(self, route_request : Optional[RouteRequest] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Get turn by turn routing instructions between two or more locations.  # noqa: E501

        The route (turn-by-turn) API computes routes between two or more locations. It supports a variety of tunable costing methods, and supports routing through intermediate waypoints and discontinuous multi-leg routes.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.route_with_http_info(route_request, async_req=True)
        >>> result = thread.get()

        :param route_request:
        :type route_request: RouteRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(RouteResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'route_request'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method route" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['route_request'] is not None:
            _body_params = _params['route_request']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['ApiKeyAuth']  # noqa: E501

        _response_types_map = {
            '200': "RouteResponse",
            '400': None,
            '500': None,
        }

        return self.api_client.call_api(
            '/route/v1', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def time_distance_matrix(self, matrix_request : Optional[MatrixRequest] = None, **kwargs) -> MatrixResponse:  # noqa: E501
        """Calculate a time distance matrix for a grid of start and end points.  # noqa: E501

        The time distance matrix API lets you compare travel times between a set of possible start and end points. Note that all matrix endpoints have a limit of 1000 elements, regardless of the costing/mode of travel. A matrix request with 3 inputs and 5 outputs has 3 x 5 = 15 elements. This means you could send a 100 x 10 or 20 x 50 matrix request (each having 1000 elements), but not 40 x 30 as it has 1200 elements.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.time_distance_matrix(matrix_request, async_req=True)
        >>> result = thread.get()

        :param matrix_request:
        :type matrix_request: MatrixRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: MatrixResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the time_distance_matrix_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.time_distance_matrix_with_http_info(matrix_request, **kwargs)  # noqa: E501

    @validate_arguments
    def time_distance_matrix_with_http_info(self, matrix_request : Optional[MatrixRequest] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Calculate a time distance matrix for a grid of start and end points.  # noqa: E501

        The time distance matrix API lets you compare travel times between a set of possible start and end points. Note that all matrix endpoints have a limit of 1000 elements, regardless of the costing/mode of travel. A matrix request with 3 inputs and 5 outputs has 3 x 5 = 15 elements. This means you could send a 100 x 10 or 20 x 50 matrix request (each having 1000 elements), but not 40 x 30 as it has 1200 elements.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.time_distance_matrix_with_http_info(matrix_request, async_req=True)
        >>> result = thread.get()

        :param matrix_request:
        :type matrix_request: MatrixRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(MatrixResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'matrix_request'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method time_distance_matrix" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['matrix_request'] is not None:
            _body_params = _params['matrix_request']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['ApiKeyAuth']  # noqa: E501

        _response_types_map = {
            '200': "MatrixResponse",
        }

        return self.api_client.call_api(
            '/matrix/v1', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def trace_attributes(self, trace_attributes_request : Optional[TraceAttributesRequest] = None, **kwargs) -> TraceAttributesResponse:  # noqa: E501
        """Trace the attributes of roads visited on a route.  # noqa: E501

        The trace attributes endpoint retrieves detailed information along a route, returning details for each section along the path, as well as any intersections encountered. In addition to tracing a recording route, this is great for providing just-in-time info to navigation applications, enabling them to conserve resources by omitting info like speed limits upfront that may be irrelevant if the user goes off-route.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.trace_attributes(trace_attributes_request, async_req=True)
        >>> result = thread.get()

        :param trace_attributes_request:
        :type trace_attributes_request: TraceAttributesRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: TraceAttributesResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the trace_attributes_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.trace_attributes_with_http_info(trace_attributes_request, **kwargs)  # noqa: E501

    @validate_arguments
    def trace_attributes_with_http_info(self, trace_attributes_request : Optional[TraceAttributesRequest] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Trace the attributes of roads visited on a route.  # noqa: E501

        The trace attributes endpoint retrieves detailed information along a route, returning details for each section along the path, as well as any intersections encountered. In addition to tracing a recording route, this is great for providing just-in-time info to navigation applications, enabling them to conserve resources by omitting info like speed limits upfront that may be irrelevant if the user goes off-route.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.trace_attributes_with_http_info(trace_attributes_request, async_req=True)
        >>> result = thread.get()

        :param trace_attributes_request:
        :type trace_attributes_request: TraceAttributesRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(TraceAttributesResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'trace_attributes_request'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method trace_attributes" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['trace_attributes_request'] is not None:
            _body_params = _params['trace_attributes_request']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['ApiKeyAuth']  # noqa: E501

        _response_types_map = {
            '200': "TraceAttributesResponse",
        }

        return self.api_client.call_api(
            '/trace_attributes/v1', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))



# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.4
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictBool, StrictStr, validator

class HighwayClassification(BaseModel):
    """
    HighwayClassification
    """
    internal: Optional[StrictBool] = Field(None, description="Is the edge internal to an intersection?")
    link: Optional[StrictBool] = Field(None, description="Is the edge a ramp or turn channel?")
    surface: Optional[StrictStr] = Field(None, description="A representation of the smoothness of the highway. This is used for costing and access checks based on the vehicle type.")
    use: Optional[StrictStr] = None
    classification: Optional[StrictStr] = Field(None, description="The classification/importance of the road/path. Used for a variety of purposes including fallback speed estimation and access for certain vehicle types.")
    additional_properties: Dict[str, Any] = {}
    __properties = ["internal", "link", "surface", "use", "classification"]

    @validator('surface')
    def surface_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('paved_smooth', 'paved', 'paved_rough', 'compacted', 'dirt', 'gravel', 'path', 'impassable'):
            raise ValueError("must be one of enum values ('paved_smooth', 'paved', 'paved_rough', 'compacted', 'dirt', 'gravel', 'path', 'impassable')")
        return value

    @validator('use')
    def use_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('road', 'ramp', 'turn_channel', 'track', 'driveway', 'alley', 'parking_aisle', 'emergency_access', 'drive_through', 'culdesac', 'living_street', 'service_road', 'cycleway', 'mountain_bike', 'sidewalk', 'footway', 'elevator', 'steps', 'escalator', 'path', 'pedestrian', 'bridleway', 'pedestrian_crossing', 'rest_area', 'service_area', 'other', 'rail', 'ferry', 'rail-ferry', 'bus', 'egress_connection', 'platform_connnection', 'transit_connection', 'construction'):
            raise ValueError("must be one of enum values ('road', 'ramp', 'turn_channel', 'track', 'driveway', 'alley', 'parking_aisle', 'emergency_access', 'drive_through', 'culdesac', 'living_street', 'service_road', 'cycleway', 'mountain_bike', 'sidewalk', 'footway', 'elevator', 'steps', 'escalator', 'path', 'pedestrian', 'bridleway', 'pedestrian_crossing', 'rest_area', 'service_area', 'other', 'rail', 'ferry', 'rail-ferry', 'bus', 'egress_connection', 'platform_connnection', 'transit_connection', 'construction')")
        return value

    @validator('classification')
    def classification_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('motorway', 'trunk', 'primary', 'secondary', 'tertiary', 'unclassified', 'residential', 'service_other'):
            raise ValueError("must be one of enum values ('motorway', 'trunk', 'primary', 'secondary', 'tertiary', 'unclassified', 'residential', 'service_other')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> HighwayClassification:
        """Create an instance of HighwayClassification from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> HighwayClassification:
        """Create an instance of HighwayClassification from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return HighwayClassification.parse_obj(obj)

        _obj = HighwayClassification.parse_obj({
            "internal": obj.get("internal"),
            "link": obj.get("link"),
            "surface": obj.get("surface"),
            "use": obj.get("use"),
            "classification": obj.get("classification")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj




# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.1
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import unittest
import datetime

import stadiamaps
from stadiamaps.models.trace_edge import TraceEdge  # noqa: E501
from stadiamaps.rest import ApiException

class TestTraceEdge(unittest.TestCase):
    """TraceEdge unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional):
        """Test TraceEdge
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `TraceEdge`
        """
        model = stadiamaps.models.trace_edge.TraceEdge()  # noqa: E501
        if include_optional :
            return TraceEdge(
                names = [
                    ''
                    ], 
                length = 1.337, 
                speed = 56, 
                road_class = 'motorway', 
                begin_heading = 56, 
                end_heading = 56, 
                begin_shape_index = 56, 
                end_shape_index = 56, 
                traversability = 'forward', 
                use = 'road', 
                toll = True, 
                unpaved = True, 
                tunnel = True, 
                bridge = True, 
                roundabout = True, 
                internal_intersection = True, 
                drive_on_right = True, 
                surface = 'paved_smooth', 
                sign = stadiamaps.models.edge_sign.edgeSign(
                    exit_number = [
                        '91B'
                        ], 
                    exit_branch = [
                        'I 95 North'
                        ], 
                    exit_toward = [
                        'New York'
                        ], 
                    exit_name = [
                        'Gettysburg Pike'
                        ], ), 
                travel_mode = 'drive', 
                vehicle_type = 'car', 
                pedestrian_type = 'foot', 
                bicycle_type = 'road', 
                transit_type = 'tram', 
                id = 56, 
                way_id = 56, 
                weighted_grade = 1.337, 
                max_upward_grade = 56, 
                max_downward_grade = 56, 
                mean_elevation = 56, 
                lane_count = 56, 
                cycle_lane = 'none', 
                bicycle_network = 56, 
                sac_scale = 56, 
                sidewalk = 'left', 
                density = 56, 
                speed_limit = None, 
                truck_speed = 56, 
                truck_route = True, 
                end_node = stadiamaps.models.end_node.endNode(
                    intersecting_edges = [
                        stadiamaps.models.intersecting_edge.intersectingEdge(
                            begin_heading = 56, 
                            from_edge_name_consistency = True, 
                            to_edge_name_consistency = True, 
                            driveability = 'forward', 
                            cyclability = 'forward', 
                            walkability = 'forward', 
                            use = 'road', 
                            road_class = 'motorway', )
                        ], 
                    elapsed_time = 1.337, 
                    admin_index = 56, 
                    type = 'street_intersection', 
                    fork = True, 
                    time_zone = 'America/New_York', )
            )
        else :
            return TraceEdge(
        )
        """

    def testTraceEdge(self):
        """Test TraceEdge"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.4
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import re  # noqa: F401
import io
import warnings

from pydantic import validate_arguments, ValidationError
from typing_extensions import Annotated

from pydantic import Field, confloat, conint

from typing import Optional, Union

from stadiamaps.models.height_request import HeightRequest
from stadiamaps.models.height_response import HeightResponse
from stadiamaps.models.tz_response import TzResponse

from stadiamaps.api_client import ApiClient
from stadiamaps.api_response import ApiResponse
from stadiamaps.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class GeospatialApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_arguments
    def elevation(self, height_request : Optional[HeightRequest] = None, **kwargs) -> HeightResponse:  # noqa: E501
        """Get the elevation profile along a polyline or at a point.  # noqa: E501

        The Stadia elevation API allows you to get the elevation of any point on earth. You can pass either a simple set of points or a Google encoded polyline string. This pairs well with our routing APIs, so you can generate an elevation profile for your next bike or run.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.elevation(height_request, async_req=True)
        >>> result = thread.get()

        :param height_request:
        :type height_request: HeightRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: HeightResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the elevation_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.elevation_with_http_info(height_request, **kwargs)  # noqa: E501

    @validate_arguments
    def elevation_with_http_info(self, height_request : Optional[HeightRequest] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Get the elevation profile along a polyline or at a point.  # noqa: E501

        The Stadia elevation API allows you to get the elevation of any point on earth. You can pass either a simple set of points or a Google encoded polyline string. This pairs well with our routing APIs, so you can generate an elevation profile for your next bike or run.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.elevation_with_http_info(height_request, async_req=True)
        >>> result = thread.get()

        :param height_request:
        :type height_request: HeightRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(HeightResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'height_request'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method elevation" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['height_request'] is not None:
            _body_params = _params['height_request']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['ApiKeyAuth']  # noqa: E501

        _response_types_map = {
            '200': "HeightResponse",
        }

        return self.api_client.call_api(
            '/elevation/v1', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def tz_lookup(self, lat : Annotated[Union[confloat(le=90, ge=-90, strict=True), conint(le=90.0, ge=-90.0, strict=True)], Field(..., description="The latitude of the point you are interested in.")], lng : Annotated[Union[confloat(le=180, ge=-180, strict=True), conint(le=180.0, ge=-180.0, strict=True)], Field(..., description="The longitude of the point you are interested in.")], timestamp : Annotated[Optional[conint(strict=True, le=8200290900000, ge=0)], Field(description="The UNIX timestamp at which the UTC and DST offsets will be calculated. This defaults to the present time. This endpoint is not necessarily guaranteed to be accurate for timestamps that occurred in the past. Time zone geographic boundaries change over time, so if the point you are querying for was previously in a different time zone, historical results will not be accurate. If, however, the point has been in the same geographic time zone for a very long time (ex: `America/New_York`), the historical data may be accurate for 100+ years in the past (depending on how far back the IANA TZDB rules have been specified).")] = None, **kwargs) -> TzResponse:  # noqa: E501
        """Get the current time zone information for any point on earth.  # noqa: E501

        The Stadia TZ API provides time zone information, as well as information about any special offset (such as DST) in effect based on the latest IANA TZDB. Note that this API may not be accurate for timestamps in the past and does not claim to report precise nautical times in the open ocean beyond territorial waters.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.tz_lookup(lat, lng, timestamp, async_req=True)
        >>> result = thread.get()

        :param lat: The latitude of the point you are interested in. (required)
        :type lat: float
        :param lng: The longitude of the point you are interested in. (required)
        :type lng: float
        :param timestamp: The UNIX timestamp at which the UTC and DST offsets will be calculated. This defaults to the present time. This endpoint is not necessarily guaranteed to be accurate for timestamps that occurred in the past. Time zone geographic boundaries change over time, so if the point you are querying for was previously in a different time zone, historical results will not be accurate. If, however, the point has been in the same geographic time zone for a very long time (ex: `America/New_York`), the historical data may be accurate for 100+ years in the past (depending on how far back the IANA TZDB rules have been specified).
        :type timestamp: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: TzResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the tz_lookup_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.tz_lookup_with_http_info(lat, lng, timestamp, **kwargs)  # noqa: E501

    @validate_arguments
    def tz_lookup_with_http_info(self, lat : Annotated[Union[confloat(le=90, ge=-90, strict=True), conint(le=90.0, ge=-90.0, strict=True)], Field(..., description="The latitude of the point you are interested in.")], lng : Annotated[Union[confloat(le=180, ge=-180, strict=True), conint(le=180.0, ge=-180.0, strict=True)], Field(..., description="The longitude of the point you are interested in.")], timestamp : Annotated[Optional[conint(strict=True, le=8200290900000, ge=0)], Field(description="The UNIX timestamp at which the UTC and DST offsets will be calculated. This defaults to the present time. This endpoint is not necessarily guaranteed to be accurate for timestamps that occurred in the past. Time zone geographic boundaries change over time, so if the point you are querying for was previously in a different time zone, historical results will not be accurate. If, however, the point has been in the same geographic time zone for a very long time (ex: `America/New_York`), the historical data may be accurate for 100+ years in the past (depending on how far back the IANA TZDB rules have been specified).")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Get the current time zone information for any point on earth.  # noqa: E501

        The Stadia TZ API provides time zone information, as well as information about any special offset (such as DST) in effect based on the latest IANA TZDB. Note that this API may not be accurate for timestamps in the past and does not claim to report precise nautical times in the open ocean beyond territorial waters.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.tz_lookup_with_http_info(lat, lng, timestamp, async_req=True)
        >>> result = thread.get()

        :param lat: The latitude of the point you are interested in. (required)
        :type lat: float
        :param lng: The longitude of the point you are interested in. (required)
        :type lng: float
        :param timestamp: The UNIX timestamp at which the UTC and DST offsets will be calculated. This defaults to the present time. This endpoint is not necessarily guaranteed to be accurate for timestamps that occurred in the past. Time zone geographic boundaries change over time, so if the point you are querying for was previously in a different time zone, historical results will not be accurate. If, however, the point has been in the same geographic time zone for a very long time (ex: `America/New_York`), the historical data may be accurate for 100+ years in the past (depending on how far back the IANA TZDB rules have been specified).
        :type timestamp: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(TzResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'lat',
            'lng',
            'timestamp'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method tz_lookup" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('lat') is not None:  # noqa: E501
            _query_params.append(('lat', _params['lat']))

        if _params.get('lng') is not None:  # noqa: E501
            _query_params.append(('lng', _params['lng']))

        if _params.get('timestamp') is not None:  # noqa: E501
            _query_params.append(('timestamp', _params['timestamp']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['ApiKeyAuth']  # noqa: E501

        _response_types_map = {
            '200': "TzResponse",
            '404': None,
        }

        return self.api_client.call_api(
            '/tz/lookup/v1', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))



# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.4
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, StrictInt, StrictStr, conlist
from stadiamaps.models.coordinate import Coordinate
from stadiamaps.models.costing_options import CostingOptions
from stadiamaps.models.directions_options import DirectionsOptions
from stadiamaps.models.matrix_costing_model import MatrixCostingModel

class MatrixRequest(BaseModel):
    """
    MatrixRequest
    """
    id: Optional[StrictStr] = Field(None, description="An identifier to disambiguate requests (echoed by the server).")
    sources: conlist(Coordinate, min_items=1) = Field(..., description="The list of starting locations")
    targets: conlist(Coordinate, min_items=1) = Field(..., description="The list of ending locations")
    costing: MatrixCostingModel = Field(...)
    costing_options: Optional[CostingOptions] = None
    matrix_locations: Optional[StrictInt] = Field(None, description="Only applicable to one-to-many or many-to-one requests. This defaults to all locations. When specified explicitly, this option allows a partial result to be returned. This is basically equivalent to \"find the closest/best locations out of the full set.\" This can have a dramatic improvement for large requests.")
    directions_options: Optional[DirectionsOptions] = None
    additional_properties: Dict[str, Any] = {}
    __properties = ["id", "sources", "targets", "costing", "costing_options", "matrix_locations", "directions_options"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> MatrixRequest:
        """Create an instance of MatrixRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in sources (list)
        _items = []
        if self.sources:
            for _item in self.sources:
                if _item:
                    _items.append(_item.to_dict())
            _dict['sources'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in targets (list)
        _items = []
        if self.targets:
            for _item in self.targets:
                if _item:
                    _items.append(_item.to_dict())
            _dict['targets'] = _items
        # override the default output from pydantic by calling `to_dict()` of costing_options
        if self.costing_options:
            _dict['costing_options'] = self.costing_options.to_dict()
        # override the default output from pydantic by calling `to_dict()` of directions_options
        if self.directions_options:
            _dict['directions_options'] = self.directions_options.to_dict()
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> MatrixRequest:
        """Create an instance of MatrixRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return MatrixRequest.parse_obj(obj)

        _obj = MatrixRequest.parse_obj({
            "id": obj.get("id"),
            "sources": [Coordinate.from_dict(_item) for _item in obj.get("sources")] if obj.get("sources") is not None else None,
            "targets": [Coordinate.from_dict(_item) for _item in obj.get("targets")] if obj.get("targets") is not None else None,
            "costing": obj.get("costing"),
            "costing_options": CostingOptions.from_dict(obj.get("costing_options")) if obj.get("costing_options") is not None else None,
            "matrix_locations": obj.get("matrix_locations"),
            "directions_options": DirectionsOptions.from_dict(obj.get("directions_options")) if obj.get("directions_options") is not None else None
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj




# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.1
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import unittest
import datetime

import stadiamaps
from stadiamaps.models.locate_edge import LocateEdge  # noqa: E501
from stadiamaps.rest import ApiException

class TestLocateEdge(unittest.TestCase):
    """LocateEdge unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional):
        """Test LocateEdge
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `LocateEdge`
        """
        model = stadiamaps.models.locate_edge.LocateEdge()  # noqa: E501
        if include_optional :
            return LocateEdge(
                edge_id = stadiamaps.models.node_id.nodeId(
                    id = 56, 
                    value = 56, 
                    tile_id = 56, 
                    level = 56, ), 
                correlated_lat = 1.337, 
                correlated_lon = 1.337, 
                percent_along = 1.337, 
                side_of_street = 'left', 
                linear_reference = '', 
                outbound_reach = 56, 
                heading = 1.337, 
                inbound_reach = 56, 
                distance = 1.337, 
                predicted_speeds = [
                    56
                    ], 
                edge_info = stadiamaps.models.locate_edge_info.locateEdgeInfo(
                    mean_elevation = 1.337, 
                    shape = '', 
                    names = [
                        ''
                        ], 
                    bike_network = stadiamaps.models.bike_network.bikeNetwork(
                        mountain = True, 
                        local = True, 
                        regional = True, 
                        national = True, ), 
                    way_id = 56, ), 
                edge = stadiamaps.models.locate_detailed_edge.locateDetailedEdge(
                    sidewalk_left = True, 
                    sidewalk_right = True, 
                    lane_count = 56, 
                    stop_sign = True, 
                    sac_scale = 'none', 
                    yield_sign = True, 
                    not_thru = True, 
                    forward = True, 
                    end_node = stadiamaps.models.node_id.nodeId(
                        id = 56, 
                        value = 56, 
                        tile_id = 56, 
                        level = 56, ), 
                    truck_route = True, 
                    speeds = stadiamaps.models.speeds.speeds(
                        predicted = True, 
                        constrained_flow = 56, 
                        free_flow = 56, 
                        type = 'classified', 
                        default = 56, ), 
                    bike_network = True, 
                    round_about = True, 
                    traffic_signal = True, 
                    access_restriction = True, 
                    destination_only = True, 
                    geo_attributes = stadiamaps.models.geo_attributes.geoAttributes(
                        curvature = 56, 
                        max_down_slope = 1.337, 
                        max_up_slope = 1.337, 
                        weighted_grade = 1.337, 
                        length = 56, ), 
                    start_restriction = stadiamaps.models.restrictions.restrictions(
                        golf_cart = True, 
                        truck = True, 
                        pedestrian = True, 
                        wheelchair = True, 
                        taxi = True, 
                        hov = True, 
                        emergency = True, 
                        motorcycle = True, 
                        car = True, 
                        moped = True, 
                        bus = True, 
                        bicycle = True, ), 
                    cycle_lane = 'none', 
                    end_restriction = stadiamaps.models.restrictions.restrictions(
                        golf_cart = True, 
                        truck = True, 
                        pedestrian = True, 
                        wheelchair = True, 
                        taxi = True, 
                        hov = True, 
                        emergency = True, 
                        motorcycle = True, 
                        car = True, 
                        moped = True, 
                        bus = True, 
                        bicycle = True, ), 
                    seasonal = True, 
                    country_crossing = True, 
                    part_of_complex_restriction = True, 
                    has_sign = True, 
                    access = , 
                    bridge = True, 
                    classification = stadiamaps.models.highway_classification.highwayClassification(
                        internal = True, 
                        link = True, 
                        surface = 'paved_smooth', 
                        use = 'road', ), 
                    toll = True, 
                    tunnel = True, ), 
                warnings = [
                    ''
                    ]
            )
        else :
            return LocateEdge(
        )
        """

    def testLocateEdge(self):
        """Test LocateEdge"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.1
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import unittest
import datetime

import stadiamaps
from stadiamaps.models.locate_detailed_edge import LocateDetailedEdge  # noqa: E501
from stadiamaps.rest import ApiException

class TestLocateDetailedEdge(unittest.TestCase):
    """LocateDetailedEdge unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional):
        """Test LocateDetailedEdge
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `LocateDetailedEdge`
        """
        model = stadiamaps.models.locate_detailed_edge.LocateDetailedEdge()  # noqa: E501
        if include_optional :
            return LocateDetailedEdge(
                sidewalk_left = True, 
                sidewalk_right = True, 
                lane_count = 56, 
                stop_sign = True, 
                sac_scale = 'none', 
                yield_sign = True, 
                not_thru = True, 
                forward = True, 
                end_node = stadiamaps.models.node_id.nodeId(
                    id = 56, 
                    value = 56, 
                    tile_id = 56, 
                    level = 56, ), 
                truck_route = True, 
                speeds = stadiamaps.models.speeds.speeds(
                    predicted = True, 
                    constrained_flow = 56, 
                    free_flow = 56, 
                    type = 'classified', 
                    default = 56, ), 
                bike_network = True, 
                round_about = True, 
                traffic_signal = True, 
                access_restriction = True, 
                destination_only = True, 
                geo_attributes = stadiamaps.models.geo_attributes.geoAttributes(
                    curvature = 56, 
                    max_down_slope = 1.337, 
                    max_up_slope = 1.337, 
                    weighted_grade = 1.337, 
                    length = 56, ), 
                start_restriction = stadiamaps.models.restrictions.restrictions(
                    golf_cart = True, 
                    truck = True, 
                    pedestrian = True, 
                    wheelchair = True, 
                    taxi = True, 
                    hov = True, 
                    emergency = True, 
                    motorcycle = True, 
                    car = True, 
                    moped = True, 
                    bus = True, 
                    bicycle = True, ), 
                cycle_lane = 'none', 
                end_restriction = stadiamaps.models.restrictions.restrictions(
                    golf_cart = True, 
                    truck = True, 
                    pedestrian = True, 
                    wheelchair = True, 
                    taxi = True, 
                    hov = True, 
                    emergency = True, 
                    motorcycle = True, 
                    car = True, 
                    moped = True, 
                    bus = True, 
                    bicycle = True, ), 
                seasonal = True, 
                country_crossing = True, 
                part_of_complex_restriction = True, 
                has_sign = True, 
                access = stadiamaps.models.restrictions.restrictions(
                    golf_cart = True, 
                    truck = True, 
                    pedestrian = True, 
                    wheelchair = True, 
                    taxi = True, 
                    hov = True, 
                    emergency = True, 
                    motorcycle = True, 
                    car = True, 
                    moped = True, 
                    bus = True, 
                    bicycle = True, ), 
                bridge = True, 
                classification = stadiamaps.models.highway_classification.highwayClassification(
                    internal = True, 
                    link = True, 
                    surface = 'paved_smooth', 
                    use = 'road', ), 
                toll = True, 
                tunnel = True
            )
        else :
            return LocateDetailedEdge(
        )
        """

    def testLocateDetailedEdge(self):
        """Test LocateDetailedEdge"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.4
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional, Union
from pydantic import BaseModel, Field, StrictFloat, StrictInt

class MapMatchTraceOptions(BaseModel):
    """
    MapMatchTraceOptions
    """
    search_radius: Optional[StrictInt] = Field(None, description="The search radius, in meters, when trying to match each trace point.")
    gps_accuracy: Optional[Union[StrictFloat, StrictInt]] = Field(None, description="The accuracy of the GPS, in meters.")
    breakage_distance: Optional[Union[StrictFloat, StrictInt]] = Field(None, description="The breaking distance, in meters, between trace points.")
    interpolation_distance: Optional[Union[StrictFloat, StrictInt]] = Field(None, description="The interpolation distance, in meters, beyond which trace points are merged together.")
    turn_penalty_factor: Optional[StrictInt] = Field(None, description="Penalizes turns from one road segment to next. For a pedestrian trace, you may see a back-and-forth motion along the streets of your path with the default settings. Try increasing the turn penalty factor to 500 to reduce jitter in the output. Note that if GPS accuracy is already good, increasing this above the default will usually negatively affect the quality of map matching.")
    additional_properties: Dict[str, Any] = {}
    __properties = ["search_radius", "gps_accuracy", "breakage_distance", "interpolation_distance", "turn_penalty_factor"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> MapMatchTraceOptions:
        """Create an instance of MapMatchTraceOptions from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> MapMatchTraceOptions:
        """Create an instance of MapMatchTraceOptions from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return MapMatchTraceOptions.parse_obj(obj)

        _obj = MapMatchTraceOptions.parse_obj({
            "search_radius": obj.get("search_radius"),
            "gps_accuracy": obj.get("gps_accuracy"),
            "breakage_distance": obj.get("breakage_distance"),
            "interpolation_distance": obj.get("interpolation_distance"),
            "turn_penalty_factor": obj.get("turn_penalty_factor")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj




# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.4
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel
from stadiamaps.models.auto_costing_options import AutoCostingOptions
from stadiamaps.models.bicycle_costing_options import BicycleCostingOptions
from stadiamaps.models.motor_scooter_costing_options import MotorScooterCostingOptions
from stadiamaps.models.motorcycle_costing_options import MotorcycleCostingOptions
from stadiamaps.models.pedestrian_costing_options import PedestrianCostingOptions
from stadiamaps.models.truck_costing_options import TruckCostingOptions

class CostingOptions(BaseModel):
    """
    CostingOptions
    """
    auto: Optional[AutoCostingOptions] = None
    bus: Optional[AutoCostingOptions] = None
    taxi: Optional[AutoCostingOptions] = None
    truck: Optional[TruckCostingOptions] = None
    bicycle: Optional[BicycleCostingOptions] = None
    motor_scooter: Optional[MotorScooterCostingOptions] = None
    motorcycle: Optional[MotorcycleCostingOptions] = None
    pedestrian: Optional[PedestrianCostingOptions] = None
    additional_properties: Dict[str, Any] = {}
    __properties = ["auto", "bus", "taxi", "truck", "bicycle", "motor_scooter", "motorcycle", "pedestrian"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CostingOptions:
        """Create an instance of CostingOptions from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of auto
        if self.auto:
            _dict['auto'] = self.auto.to_dict()
        # override the default output from pydantic by calling `to_dict()` of bus
        if self.bus:
            _dict['bus'] = self.bus.to_dict()
        # override the default output from pydantic by calling `to_dict()` of taxi
        if self.taxi:
            _dict['taxi'] = self.taxi.to_dict()
        # override the default output from pydantic by calling `to_dict()` of truck
        if self.truck:
            _dict['truck'] = self.truck.to_dict()
        # override the default output from pydantic by calling `to_dict()` of bicycle
        if self.bicycle:
            _dict['bicycle'] = self.bicycle.to_dict()
        # override the default output from pydantic by calling `to_dict()` of motor_scooter
        if self.motor_scooter:
            _dict['motor_scooter'] = self.motor_scooter.to_dict()
        # override the default output from pydantic by calling `to_dict()` of motorcycle
        if self.motorcycle:
            _dict['motorcycle'] = self.motorcycle.to_dict()
        # override the default output from pydantic by calling `to_dict()` of pedestrian
        if self.pedestrian:
            _dict['pedestrian'] = self.pedestrian.to_dict()
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> CostingOptions:
        """Create an instance of CostingOptions from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return CostingOptions.parse_obj(obj)

        _obj = CostingOptions.parse_obj({
            "auto": AutoCostingOptions.from_dict(obj.get("auto")) if obj.get("auto") is not None else None,
            "bus": AutoCostingOptions.from_dict(obj.get("bus")) if obj.get("bus") is not None else None,
            "taxi": AutoCostingOptions.from_dict(obj.get("taxi")) if obj.get("taxi") is not None else None,
            "truck": TruckCostingOptions.from_dict(obj.get("truck")) if obj.get("truck") is not None else None,
            "bicycle": BicycleCostingOptions.from_dict(obj.get("bicycle")) if obj.get("bicycle") is not None else None,
            "motor_scooter": MotorScooterCostingOptions.from_dict(obj.get("motor_scooter")) if obj.get("motor_scooter") is not None else None,
            "motorcycle": MotorcycleCostingOptions.from_dict(obj.get("motorcycle")) if obj.get("motorcycle") is not None else None,
            "pedestrian": PedestrianCostingOptions.from_dict(obj.get("pedestrian")) if obj.get("pedestrian") is not None else None
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj




# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.4
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional, Union
from pydantic import BaseModel, Field, StrictInt, StrictStr, confloat, conint, conlist
from stadiamaps.models.admin_region import AdminRegion
from stadiamaps.models.matched_point import MatchedPoint
from stadiamaps.models.trace_edge import TraceEdge

class TraceAttributesBaseResponse(BaseModel):
    """
    TraceAttributesBaseResponse
    """
    edges: Optional[conlist(TraceEdge)] = Field(None, description="The list of edges matched along the path.")
    admins: Optional[conlist(AdminRegion)] = Field(None, description="The set of administrative regions matched along the path. Rather than repeating this information for every end node, the admins in this list are referenced by index.")
    matched_points: Optional[conlist(MatchedPoint)] = Field(None, description="List of match results when using the map_snap shape match algorithm. There is a one-to-one correspondence with the input set of latitude, longitude coordinates and this list of match results.")
    osm_changeset: Optional[StrictInt] = None
    shape: Optional[StrictStr] = Field(None, description="The encoded polyline (https://developers.google.com/maps/documentation/utilities/polylinealgorithm) of the matched path.")
    confidence_score: Optional[Union[confloat(le=1, ge=0, strict=True), conint(le=1, ge=0, strict=True)]] = None
    additional_properties: Dict[str, Any] = {}
    __properties = ["edges", "admins", "matched_points", "osm_changeset", "shape", "confidence_score"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> TraceAttributesBaseResponse:
        """Create an instance of TraceAttributesBaseResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in edges (list)
        _items = []
        if self.edges:
            for _item in self.edges:
                if _item:
                    _items.append(_item.to_dict())
            _dict['edges'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in admins (list)
        _items = []
        if self.admins:
            for _item in self.admins:
                if _item:
                    _items.append(_item.to_dict())
            _dict['admins'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in matched_points (list)
        _items = []
        if self.matched_points:
            for _item in self.matched_points:
                if _item:
                    _items.append(_item.to_dict())
            _dict['matched_points'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> TraceAttributesBaseResponse:
        """Create an instance of TraceAttributesBaseResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return TraceAttributesBaseResponse.parse_obj(obj)

        _obj = TraceAttributesBaseResponse.parse_obj({
            "edges": [TraceEdge.from_dict(_item) for _item in obj.get("edges")] if obj.get("edges") is not None else None,
            "admins": [AdminRegion.from_dict(_item) for _item in obj.get("admins")] if obj.get("admins") is not None else None,
            "matched_points": [MatchedPoint.from_dict(_item) for _item in obj.get("matched_points")] if obj.get("matched_points") is not None else None,
            "osm_changeset": obj.get("osm_changeset"),
            "shape": obj.get("shape"),
            "confidence_score": obj.get("confidence_score")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj




# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.4
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional, Union
from pydantic import BaseModel, Field, StrictBool, StrictFloat, StrictInt, StrictStr, confloat, conint, conlist
from stadiamaps.models.contour import Contour
from stadiamaps.models.coordinate import Coordinate
from stadiamaps.models.costing_options import CostingOptions
from stadiamaps.models.isochrone_costing_model import IsochroneCostingModel

class IsochroneRequest(BaseModel):
    """
    IsochroneRequest
    """
    id: Optional[StrictStr] = Field(None, description="An identifier to disambiguate requests (echoed by the server).")
    locations: conlist(Coordinate) = Field(...)
    costing: IsochroneCostingModel = Field(...)
    costing_options: Optional[CostingOptions] = None
    contours: conlist(Contour, max_items=4, min_items=1) = Field(...)
    polygons: Optional[StrictBool] = Field(False, description="If true, the generated GeoJSON will use polygons. The default is to use LineStrings. Polygon output makes it easier to render overlapping areas in some visualization tools (such as MapLibre renderers).")
    denoise: Optional[Union[confloat(le=1, ge=0, strict=True), conint(le=1, ge=0, strict=True)]] = Field(1, description="A value in the range [0, 1] which will be used to smooth out or remove smaller contours. A value of 1 will only return the largest contour for a given time value. A value of 0.5 drops any contours that are less than half the area of the largest contour in the set of contours for that same time value.")
    generalize: Optional[Union[StrictFloat, StrictInt]] = Field(200.0, description="The value in meters to be used as a tolerance for Douglas-Peucker generalization.")
    show_locations: Optional[StrictBool] = Field(False, description="If true, then the output GeoJSON will include the input locations as two MultiPoint features: one for the exact input coordinates, and a second for the route network node location that the point was snapped to.")
    additional_properties: Dict[str, Any] = {}
    __properties = ["id", "locations", "costing", "costing_options", "contours", "polygons", "denoise", "generalize", "show_locations"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> IsochroneRequest:
        """Create an instance of IsochroneRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in locations (list)
        _items = []
        if self.locations:
            for _item in self.locations:
                if _item:
                    _items.append(_item.to_dict())
            _dict['locations'] = _items
        # override the default output from pydantic by calling `to_dict()` of costing_options
        if self.costing_options:
            _dict['costing_options'] = self.costing_options.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in contours (list)
        _items = []
        if self.contours:
            for _item in self.contours:
                if _item:
                    _items.append(_item.to_dict())
            _dict['contours'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> IsochroneRequest:
        """Create an instance of IsochroneRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return IsochroneRequest.parse_obj(obj)

        _obj = IsochroneRequest.parse_obj({
            "id": obj.get("id"),
            "locations": [Coordinate.from_dict(_item) for _item in obj.get("locations")] if obj.get("locations") is not None else None,
            "costing": obj.get("costing"),
            "costing_options": CostingOptions.from_dict(obj.get("costing_options")) if obj.get("costing_options") is not None else None,
            "contours": [Contour.from_dict(_item) for _item in obj.get("contours")] if obj.get("contours") is not None else None,
            "polygons": obj.get("polygons") if obj.get("polygons") is not None else False,
            "denoise": obj.get("denoise") if obj.get("denoise") is not None else 1,
            "generalize": obj.get("generalize") if obj.get("generalize") is not None else 200.0,
            "show_locations": obj.get("show_locations") if obj.get("show_locations") is not None else False
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj




# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.4
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional, Union
from pydantic import BaseModel, Field, StrictFloat, StrictInt, StrictStr, conlist, validator
from stadiamaps.models.geo_json_point import GeoJSONPoint
from stadiamaps.models.pelias_geo_json_properties import PeliasGeoJSONProperties

class PeliasGeoJSONFeature(BaseModel):
    """
    PeliasGeoJSONFeature
    """
    type: StrictStr = Field(...)
    geometry: GeoJSONPoint = Field(...)
    properties: Optional[PeliasGeoJSONProperties] = None
    bbox: Optional[conlist(Union[StrictFloat, StrictInt], max_items=4, min_items=4)] = Field(None, description="An array of 4 floating point numbers representing the (W, S, E, N) extremes of the features found.")
    id: Optional[StrictStr] = None
    additional_properties: Dict[str, Any] = {}
    __properties = ["type", "geometry", "properties", "bbox", "id"]

    @validator('type')
    def type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('Feature'):
            raise ValueError("must be one of enum values ('Feature')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> PeliasGeoJSONFeature:
        """Create an instance of PeliasGeoJSONFeature from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of geometry
        if self.geometry:
            _dict['geometry'] = self.geometry.to_dict()
        # override the default output from pydantic by calling `to_dict()` of properties
        if self.properties:
            _dict['properties'] = self.properties.to_dict()
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> PeliasGeoJSONFeature:
        """Create an instance of PeliasGeoJSONFeature from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return PeliasGeoJSONFeature.parse_obj(obj)

        _obj = PeliasGeoJSONFeature.parse_obj({
            "type": obj.get("type"),
            "geometry": GeoJSONPoint.from_dict(obj.get("geometry")) if obj.get("geometry") is not None else None,
            "properties": PeliasGeoJSONProperties.from_dict(obj.get("properties")) if obj.get("properties") is not None else None,
            "bbox": obj.get("bbox"),
            "id": obj.get("id")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj




# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.4
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class Traversability(str, Enum):
    """
    The directions in which the edge is traversable.
    """

    """
    allowed enum values
    """
    FORWARD = 'forward'
    BACKWARD = 'backward'
    BOTH = 'both'

    @classmethod
    def from_json(cls, json_str: str) -> Traversability:
        """Create an instance of Traversability from a JSON string"""
        return Traversability(json.loads(json_str))





# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.4
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictStr, validator
from stadiamaps.models.pelias_geo_json_properties_addendum import PeliasGeoJSONPropertiesAddendum
from stadiamaps.models.pelias_layer import PeliasLayer

class PeliasGeoJSONProperties(BaseModel):
    """
    PeliasGeoJSONProperties
    """
    gid: Optional[StrictStr] = Field(None, description="A scoped GID for this result. This can be passed to the place endpoint. Note that these are not always stable. For OSM, Geonames, and Who's on First, these are usually stable, but for other sources like OSM, no stability is guaranteed.")
    source_id: Optional[StrictStr] = Field(None, description="An ID referencing the original data source (specified via source) for the result. These IDs are specific to the source that they originated from. For example, in the case of OSM, these typically look like way/123 or point/123.")
    label: Optional[StrictStr] = Field(None, description="A full, human-readable label. However, you may not necessarily want to use this; be sure to read the docs for name, locality, and region before making a decision. This field is mostly localized. The order of components is generally locally correct (ex: for an address in South Korea, the house number appears after the street name). However, components will use a request language equivalent if one exists (ex: Seoul instead of 서울 if lang=en).")
    layer: Optional[PeliasLayer] = None
    name: Optional[StrictStr] = Field(None, description="The name of the place, the street address including house number, or label of similar relevance. If your app is localized to a specific region, you may get better display results by combining name, locality OR region (or neither?), and postal code together in the local format. Experiment with what works best for your use case.")
    accuracy: Optional[StrictStr] = Field(None, description="The accuracy of the geographic coordinates in the result. This value is a property of the result itself and won't change based on the query. A point result means that the record can reasonably be represented by a single geographic point. Addresses, venues, or interpolated addresses usually have point accuracy. Larger areas, such as a city or country, cannot be represented by a single point, so a centroid is given instead.")
    addendum: Optional[PeliasGeoJSONPropertiesAddendum] = None
    continent: Optional[StrictStr] = None
    continent_gid: Optional[StrictStr] = None
    country: Optional[StrictStr] = None
    country_gid: Optional[StrictStr] = None
    neighbourhood: Optional[StrictStr] = None
    neighbourhood_gid: Optional[StrictStr] = None
    borough: Optional[StrictStr] = None
    borough_gid: Optional[StrictStr] = None
    postalcode: Optional[StrictStr] = None
    street: Optional[StrictStr] = None
    housenumber: Optional[StrictStr] = None
    locality: Optional[StrictStr] = Field(None, description="The city, village, town, etc. that the place / address is part of. Note that values may not always match postal or local conventions perfectly.")
    locality_gid: Optional[StrictStr] = None
    county: Optional[StrictStr] = Field(None, description="Administrative divisions between localities and regions. Useful for disambiguating nearby results with similar names.")
    region: Optional[StrictStr] = Field(None, description="Typically the first administrative division within a country. For example, a US state or a Canadian province.")
    region_a: Optional[StrictStr] = Field(None, description="The abbreviation for the region.")
    additional_properties: Dict[str, Any] = {}
    __properties = ["gid", "source_id", "label", "layer", "name", "accuracy", "addendum", "continent", "continent_gid", "country", "country_gid", "neighbourhood", "neighbourhood_gid", "borough", "borough_gid", "postalcode", "street", "housenumber", "locality", "locality_gid", "county", "region", "region_a"]

    @validator('accuracy')
    def accuracy_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('point', 'centroid'):
            raise ValueError("must be one of enum values ('point', 'centroid')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> PeliasGeoJSONProperties:
        """Create an instance of PeliasGeoJSONProperties from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of addendum
        if self.addendum:
            _dict['addendum'] = self.addendum.to_dict()
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> PeliasGeoJSONProperties:
        """Create an instance of PeliasGeoJSONProperties from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return PeliasGeoJSONProperties.parse_obj(obj)

        _obj = PeliasGeoJSONProperties.parse_obj({
            "gid": obj.get("gid"),
            "source_id": obj.get("source_id"),
            "label": obj.get("label"),
            "layer": obj.get("layer"),
            "name": obj.get("name"),
            "accuracy": obj.get("accuracy"),
            "addendum": PeliasGeoJSONPropertiesAddendum.from_dict(obj.get("addendum")) if obj.get("addendum") is not None else None,
            "continent": obj.get("continent"),
            "continent_gid": obj.get("continent_gid"),
            "country": obj.get("country"),
            "country_gid": obj.get("country_gid"),
            "neighbourhood": obj.get("neighbourhood"),
            "neighbourhood_gid": obj.get("neighbourhood_gid"),
            "borough": obj.get("borough"),
            "borough_gid": obj.get("borough_gid"),
            "postalcode": obj.get("postalcode"),
            "street": obj.get("street"),
            "housenumber": obj.get("housenumber"),
            "locality": obj.get("locality"),
            "locality_gid": obj.get("locality_gid"),
            "county": obj.get("county"),
            "region": obj.get("region"),
            "region_a": obj.get("region_a")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj




# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.4
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, StrictStr, conlist
from stadiamaps.models.coordinate import Coordinate
from stadiamaps.models.matrix_distance import MatrixDistance
from stadiamaps.models.valhalla_long_units import ValhallaLongUnits
from stadiamaps.models.warning import Warning

class MatrixResponse(BaseModel):
    """
    MatrixResponse
    """
    id: Optional[StrictStr] = Field(None, description="An identifier to disambiguate requests (echoed by the server).")
    sources: conlist(conlist(Coordinate), min_items=1) = Field(..., description="The list of starting locations")
    targets: conlist(conlist(Coordinate), min_items=1) = Field(..., description="The list of ending locations")
    sources_to_targets: conlist(conlist(MatrixDistance), min_items=1) = Field(..., description="The matrix of starting and ending locations, along with the computed distance and travel time. The array is row-ordered. This means that the time and distance from the first location to all others forms the first row of the array, followed by the time and distance from the second source location to all target locations, etc.")
    warnings: Optional[conlist(Warning)] = None
    units: ValhallaLongUnits = Field(...)
    additional_properties: Dict[str, Any] = {}
    __properties = ["id", "sources", "targets", "sources_to_targets", "warnings", "units"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> MatrixResponse:
        """Create an instance of MatrixResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in sources (list)
        _items = []
        if self.sources:
            for _item in self.sources:
                if _item:
                    _items.append(_item.to_dict())
            _dict['sources'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in targets (list)
        _items = []
        if self.targets:
            for _item in self.targets:
                if _item:
                    _items.append(_item.to_dict())
            _dict['targets'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in sources_to_targets (list)
        _items = []
        if self.sources_to_targets:
            for _item in self.sources_to_targets:
                if _item:
                    _items.append(_item.to_dict())
            _dict['sources_to_targets'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in warnings (list)
        _items = []
        if self.warnings:
            for _item in self.warnings:
                if _item:
                    _items.append(_item.to_dict())
            _dict['warnings'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> MatrixResponse:
        """Create an instance of MatrixResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return MatrixResponse.parse_obj(obj)

        _obj = MatrixResponse.parse_obj({
            "id": obj.get("id"),
            "sources": [[Coordinate.from_dict(_item) for _item in _items] for _items in obj.get("sources")] if obj.get("sources") is not None else None,
            "targets": [[Coordinate.from_dict(_item) for _item in _items] for _items in obj.get("targets")] if obj.get("targets") is not None else None,
            "sources_to_targets": [[MatrixDistance.from_dict(_item) for _item in _items] for _items in obj.get("sources_to_targets")] if obj.get("sources_to_targets") is not None else None,
            "warnings": [Warning.from_dict(_item) for _item in obj.get("warnings")] if obj.get("warnings") is not None else None,
            "units": obj.get("units")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj




# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.4
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class NodeType(str, Enum):
    """
    NodeType
    """

    """
    allowed enum values
    """
    STREET_INTERSECTION = 'street_intersection'
    GATE = 'gate'
    BOLLARD = 'bollard'
    TOLL_BOOTH = 'toll_booth'
    MULTI_USE_TRANSIT_STOP = 'multi_use_transit_stop'
    BIKE_SHARE = 'bike_share'
    PARKING = 'parking'
    MOTOR_WAY_JUNCTION = 'motor_way_junction'
    BORDER_CONTROL = 'border_control'

    @classmethod
    def from_json(cls, json_str: str) -> NodeType:
        """Create an instance of NodeType from a JSON string"""
        return NodeType(json.loads(json_str))





# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.4
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class IsochroneCostingModel(str, Enum):
    """
    IsochroneCostingModel
    """

    """
    allowed enum values
    """
    AUTO = 'auto'
    BICYCLE = 'bicycle'
    PEDESTRIAN = 'pedestrian'

    @classmethod
    def from_json(cls, json_str: str) -> IsochroneCostingModel:
        """Create an instance of IsochroneCostingModel from a JSON string"""
        return IsochroneCostingModel(json.loads(json_str))





# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.4
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional, Union
from pydantic import BaseModel, Field, StrictBool, StrictFloat, StrictInt, StrictStr, conint, validator
from stadiamaps.models.routing_waypoint_all_of_search_filter import RoutingWaypointAllOfSearchFilter

class RoutingWaypoint(BaseModel):
    """
    RoutingWaypoint
    """
    lat: Union[StrictFloat, StrictInt] = Field(..., description="The latitude of a point in the shape.")
    lon: Union[StrictFloat, StrictInt] = Field(..., description="The longitude of a point in the shape.")
    type: Optional[StrictStr] = Field('break', description="A `break` represents the start or end of a leg, and allows reversals. A `through` location is an intermediate waypoint that must be visited between `break`s, but at which reversals are not allowed. A `via` is similar to a `through` except that reversals are allowed. A `break_through` is similar to a `break` in that it can be the start/end of a leg, but does not allow reversals.")
    heading: Optional[conint(strict=True, le=360, ge=0)] = Field(None, description="The preferred direction of travel when starting the route, in integer clockwise degrees from north. North is 0, south is 180, east is 90, and west is 270.")
    heading_tolerance: Optional[conint(strict=True, le=360, ge=0)] = Field(60, description="The tolerance (in degrees) determining whether a street is considered the same direction.")
    minimum_reachability: Optional[conint(strict=True, ge=0)] = Field(50, description="The minimum number of nodes that must be reachable for a given edge to consider that edge as belonging to a connected region. If a candidate edge has fewer connections, it will be considered a disconnected island.")
    radius: Optional[conint(strict=True, ge=0)] = Field(0, description="The distance (in meters) to look for candidate edges around the location for purposes of snapping locations to the route graph. If there are no candidates within this distance, the closest candidate within a reasonable search distance will be used. This is subject to clamping by internal limits.")
    rank_candidates: Optional[StrictBool] = Field(True, description="If true, candidates will be ranked according to their distance from the target location as well as other factors. If false, candidates will only be ranked using their distance from the target.")
    preferred_side: Optional[StrictStr] = Field(None, description="If the location is not offset from the road centerline or is closest to an intersection, this option has no effect. Otherwise, the preferred side of street is used to determine whether or not the location should be visited from the same, opposite or either side of the road with respect to the side of the road the given locale drives on.")
    node_snap_tolerance: Optional[conint(strict=True, ge=0)] = Field(5, description="During edge correlation this is the tolerance (in meters) used to determine whether or not to snap to the intersection rather than along the street, if the snap location is within this distance from the intersection, the intersection is used instead.")
    street_side_tolerance: Optional[conint(strict=True, ge=0)] = Field(5, description="A tolerance in meters from the edge centerline used for determining the side of the street that the location is on. If the distance to the centerline is less than this tolerance, no side will be inferred. Otherwise, the left or right side will be selected depending on the direction of travel.")
    street_side_max_distance: Optional[conint(strict=True, ge=0)] = Field(1000, description="A tolerance in meters from the edge centerline used for determining the side of the street that the location is on. If the distance to the centerline is greater than this tolerance, no side will be inferred. Otherwise, the left or right side will be selected depending on the direction of travel.")
    search_filter: Optional[RoutingWaypointAllOfSearchFilter] = None
    additional_properties: Dict[str, Any] = {}
    __properties = ["lat", "lon", "type", "heading", "heading_tolerance", "minimum_reachability", "radius", "rank_candidates", "preferred_side", "node_snap_tolerance", "street_side_tolerance", "street_side_max_distance", "search_filter"]

    @validator('type')
    def type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('break', 'through', 'via', 'break_through'):
            raise ValueError("must be one of enum values ('break', 'through', 'via', 'break_through')")
        return value

    @validator('preferred_side')
    def preferred_side_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('same', 'opposite', 'either'):
            raise ValueError("must be one of enum values ('same', 'opposite', 'either')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> RoutingWaypoint:
        """Create an instance of RoutingWaypoint from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of search_filter
        if self.search_filter:
            _dict['search_filter'] = self.search_filter.to_dict()
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> RoutingWaypoint:
        """Create an instance of RoutingWaypoint from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return RoutingWaypoint.parse_obj(obj)

        _obj = RoutingWaypoint.parse_obj({
            "lat": obj.get("lat"),
            "lon": obj.get("lon"),
            "type": obj.get("type") if obj.get("type") is not None else 'break',
            "heading": obj.get("heading"),
            "heading_tolerance": obj.get("heading_tolerance") if obj.get("heading_tolerance") is not None else 60,
            "minimum_reachability": obj.get("minimum_reachability") if obj.get("minimum_reachability") is not None else 50,
            "radius": obj.get("radius") if obj.get("radius") is not None else 0,
            "rank_candidates": obj.get("rank_candidates") if obj.get("rank_candidates") is not None else True,
            "preferred_side": obj.get("preferred_side"),
            "node_snap_tolerance": obj.get("node_snap_tolerance") if obj.get("node_snap_tolerance") is not None else 5,
            "street_side_tolerance": obj.get("street_side_tolerance") if obj.get("street_side_tolerance") is not None else 5,
            "street_side_max_distance": obj.get("street_side_max_distance") if obj.get("street_side_max_distance") is not None else 1000,
            "search_filter": RoutingWaypointAllOfSearchFilter.from_dict(obj.get("search_filter")) if obj.get("search_filter") is not None else None
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj




# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.4
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from setuptools import setup, find_packages  # noqa: H301

# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools
NAME = "stadiamaps"
VERSION = "1.0.5"
PYTHON_REQUIRES = ">=3.7"
REQUIRES = [
    "urllib3 >= 1.25.3",
    "python-dateutil",
    "pydantic >= 1.10.5, < 2",
    "aenum"
]

setup(
    name=NAME,
    version=VERSION,
    description="Stadia Maps Geospatial APIs",
    author="Stadia Maps Support",
    author_email="support@stadiamaps.com",
    url="https://github.com/stadiamaps/stadiamaps-api-py",
    keywords=["OpenAPI", "OpenAPI-Generator", "Stadia Maps Geospatial APIs"],
    install_requires=REQUIRES,
    packages=find_packages(exclude=["test", "tests"]),
    include_package_data=True,
    long_description_content_type='text/markdown',
    long_description="""
# Stadia Maps Python API Client

The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.

For more information about the API, please visit [https://docs.stadiamaps.com](https://docs.stadiamaps.com)

## Installation & Usage
### pip install

```shell
pip install stadiamaps
```

### Setuptools

Install via [Setuptools](http://pypi.python.org/pypi/setuptools).

```shell
python setup.py install --user
```
(or `sudo python setup.py install` to install the package for all users)

### Tests

Execute `pytest` to run the tests. These are run automatically via CI.

## Getting Started

Please follow the [installation procedure](#installation--usage) and then run the following:

```python
import os
import stadiamaps
from stadiamaps.rest import ApiException
from pprint import pprint

# Defining the host is optional and defaults to https://api.stadiamaps.com
# You can also use our EU endpoint to keep traffic within the EU like so:
# configuration = stadiamaps.Configuration(host="https://api-eu.stadiamaps.com")
# See configuration.py for a list of all supported configuration parameters.
configuration = stadiamaps.Configuration()

# Configure API key authorization. This example assumes it is injected via an environment
# variable.
configuration.api_key['ApiKeyAuth'] = os.environ["API_KEY"]

# Enter a context with an instance of the API client
with stadiamaps.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = stadiamaps.GeocodingApi(api_client)
    text = "Põhja pst 27a" # str | The place name (address, venue name, etc.) to search for.

    try:
        # Search and geocode quickly based on partial input.
        api_response = api_instance.autocomplete(text)
        print("The response of GeocodingApi->autocomplete:\n")
        pprint(api_response)
    except ApiException as e:
        print("Exception when calling GeocodingApi->autocomplete: %s\n" % e)
```
    """
)



# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.4
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, List, Optional, Union
from pydantic import BaseModel, Field, StrictBool, StrictFloat, StrictInt, StrictStr, conlist, validator
from stadiamaps.models.edge_sign import EdgeSign
from stadiamaps.models.edge_use import EdgeUse
from stadiamaps.models.end_node import EndNode
from stadiamaps.models.road_class import RoadClass
from stadiamaps.models.travel_mode import TravelMode
from stadiamaps.models.traversability import Traversability

class TraceEdge(BaseModel):
    """
    TraceEdge
    """
    names: Optional[conlist(StrictStr)] = Field(None, description="The name(s) of the road at this edge, if any.")
    length: Optional[Union[StrictFloat, StrictInt]] = Field(None, description="The length of this edge in `units`.")
    speed: Optional[StrictInt] = Field(None, description="The speed of this edge in `units`/hr, in terms of average/free-flow speed for routing purposes. This is affected by any number of factors such as the road service, vehicle type, etc. and not just the posted speed limits.")
    road_class: Optional[RoadClass] = None
    begin_heading: Optional[StrictInt] = Field(None, description="The direction at the beginning of an edge. The units are degrees clockwise from north.")
    end_heading: Optional[StrictInt] = Field(None, description="The direction at the end of an edge. The units are degrees clockwise from north.")
    begin_shape_index: Optional[StrictInt] = Field(None, description="Index into the list of shape points for the start of the edge.")
    end_shape_index: Optional[StrictInt] = Field(None, description="Index into the list of shape points for the end of the edge.")
    traversability: Optional[Traversability] = None
    use: Optional[EdgeUse] = None
    toll: Optional[StrictBool] = Field(None, description="True if the edge has a toll.")
    unpaved: Optional[StrictBool] = Field(None, description="True if the edge has rough payment.")
    tunnel: Optional[StrictBool] = Field(None, description="True if the edge has a tunnel.")
    bridge: Optional[StrictBool] = Field(None, description="True if the edge has a bridge.")
    roundabout: Optional[StrictBool] = Field(None, description="True if the edge has a roundabout.")
    internal_intersection: Optional[StrictBool] = Field(None, description="True if the edge has an internal intersection.")
    drive_on_right: Optional[StrictBool] = Field(None, description="True if the edge is in an area where you must drive on the right side of the road.")
    surface: Optional[StrictStr] = Field(None, description="The type of surface for the edge.")
    sign: Optional[EdgeSign] = None
    travel_mode: Optional[TravelMode] = None
    vehicle_type: Optional[StrictStr] = None
    pedestrian_type: Optional[StrictStr] = None
    bicycle_type: Optional[StrictStr] = None
    transit_type: Optional[StrictStr] = None
    id: Optional[StrictInt] = None
    way_id: Optional[StrictInt] = Field(None, description="The way identifier of the edge in OSM.")
    weighted_grade: Optional[Union[StrictFloat, StrictInt]] = Field(None, description="The weighted grade factor. Valhalla manufactures a weighted grade from elevation data. It is a measure used for hill avoidance in routing - sort of a relative energy use along an edge. But since an edge in Valhalla can possibly go up and down over several hills it might not equate to what you would normally think of as grade.")
    max_upward_grade: Optional[StrictInt] = Field(None, description="The maximum upward slope. A value of 32768 indicates no elevation data is available for this edge.")
    max_downward_grade: Optional[StrictInt] = Field(None, description="The maximum downward slope. A value of 32768 indicates no elevation data is available for this edge.")
    mean_elevation: Optional[StrictInt] = Field(None, description="The mean elevation along the edge. Units are meters by default. If the `units` are specified as miles, then the mean elevation is returned in feet. A value of 32768 indicates no elevation data is available for this edge.")
    lane_count: Optional[StrictInt] = Field(None, description="The number of lanes for this edge.")
    cycle_lane: Optional[StrictStr] = Field(None, description="The type of cycle lane (if any) along this edge.")
    bicycle_network: Optional[StrictInt] = Field(None, description="The type of bicycle network, if any. This is an integer comprised of constants bitwise or'd together. For example, a route that's part of both a local and mountain network would have a value of 12. 1 - National 2 - Regional 4 - Local 8 - Mountain")
    sac_scale: Optional[StrictInt] = Field(None, description="The difficulty of the hiking trail according to the SAC scale. 0 - No Sac Scale 1 - Hiking 2 - Mountain hiking 3 - Demanding mountain hiking 4 - Alpine hiking 5 - Demanding alpine hiking 6 - Difficult alpine hiking")
    sidewalk: Optional[StrictStr] = None
    density: Optional[StrictInt] = None
    speed_limit: Optional[Any] = Field(None, description="The speed limit along the edge measured in `units`/hr. This may be either an integer or the string \"unlimited\" if speed limit data is available. If absent, there is no speed limit data available.")
    truck_speed: Optional[StrictInt] = Field(None, description="The truck speed of this edge in `units`/hr, in terms of average/free-flow speed for routing purposes. This is affected by any number of factors such as the road service, vehicle type, etc. and not just the posted speed limits.")
    truck_route: Optional[StrictBool] = Field(None, description="True if the edge is part of a truck route/network.")
    end_node: Optional[EndNode] = None
    additional_properties: Dict[str, Any] = {}
    __properties = ["names", "length", "speed", "road_class", "begin_heading", "end_heading", "begin_shape_index", "end_shape_index", "traversability", "use", "toll", "unpaved", "tunnel", "bridge", "roundabout", "internal_intersection", "drive_on_right", "surface", "sign", "travel_mode", "vehicle_type", "pedestrian_type", "bicycle_type", "transit_type", "id", "way_id", "weighted_grade", "max_upward_grade", "max_downward_grade", "mean_elevation", "lane_count", "cycle_lane", "bicycle_network", "sac_scale", "sidewalk", "density", "speed_limit", "truck_speed", "truck_route", "end_node"]

    @validator('surface')
    def surface_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('paved_smooth', 'paved', 'paved_rough', 'compacted', 'dirt', 'gravel', 'path', 'impassable'):
            raise ValueError("must be one of enum values ('paved_smooth', 'paved', 'paved_rough', 'compacted', 'dirt', 'gravel', 'path', 'impassable')")
        return value

    @validator('vehicle_type')
    def vehicle_type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('car', 'motorcycle', 'bus', 'tractor_trailer'):
            raise ValueError("must be one of enum values ('car', 'motorcycle', 'bus', 'tractor_trailer')")
        return value

    @validator('pedestrian_type')
    def pedestrian_type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('foot', 'wheelchair', 'segway'):
            raise ValueError("must be one of enum values ('foot', 'wheelchair', 'segway')")
        return value

    @validator('bicycle_type')
    def bicycle_type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('road', 'cross', 'hybrid', 'mountain'):
            raise ValueError("must be one of enum values ('road', 'cross', 'hybrid', 'mountain')")
        return value

    @validator('transit_type')
    def transit_type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('tram', 'metro', 'rail', 'bus', 'ferry', 'cable_car', 'gondola', 'funicular'):
            raise ValueError("must be one of enum values ('tram', 'metro', 'rail', 'bus', 'ferry', 'cable_car', 'gondola', 'funicular')")
        return value

    @validator('cycle_lane')
    def cycle_lane_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('none', 'shared', 'dedicated', 'separated'):
            raise ValueError("must be one of enum values ('none', 'shared', 'dedicated', 'separated')")
        return value

    @validator('sidewalk')
    def sidewalk_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('left', 'right', 'both', 'none'):
            raise ValueError("must be one of enum values ('left', 'right', 'both', 'none')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> TraceEdge:
        """Create an instance of TraceEdge from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of sign
        if self.sign:
            _dict['sign'] = self.sign.to_dict()
        # override the default output from pydantic by calling `to_dict()` of end_node
        if self.end_node:
            _dict['end_node'] = self.end_node.to_dict()
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        # set to None if speed_limit (nullable) is None
        # and __fields_set__ contains the field
        if self.speed_limit is None and "speed_limit" in self.__fields_set__:
            _dict['speed_limit'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> TraceEdge:
        """Create an instance of TraceEdge from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return TraceEdge.parse_obj(obj)

        _obj = TraceEdge.parse_obj({
            "names": obj.get("names"),
            "length": obj.get("length"),
            "speed": obj.get("speed"),
            "road_class": obj.get("road_class"),
            "begin_heading": obj.get("begin_heading"),
            "end_heading": obj.get("end_heading"),
            "begin_shape_index": obj.get("begin_shape_index"),
            "end_shape_index": obj.get("end_shape_index"),
            "traversability": obj.get("traversability"),
            "use": obj.get("use"),
            "toll": obj.get("toll"),
            "unpaved": obj.get("unpaved"),
            "tunnel": obj.get("tunnel"),
            "bridge": obj.get("bridge"),
            "roundabout": obj.get("roundabout"),
            "internal_intersection": obj.get("internal_intersection"),
            "drive_on_right": obj.get("drive_on_right"),
            "surface": obj.get("surface"),
            "sign": EdgeSign.from_dict(obj.get("sign")) if obj.get("sign") is not None else None,
            "travel_mode": obj.get("travel_mode"),
            "vehicle_type": obj.get("vehicle_type"),
            "pedestrian_type": obj.get("pedestrian_type"),
            "bicycle_type": obj.get("bicycle_type"),
            "transit_type": obj.get("transit_type"),
            "id": obj.get("id"),
            "way_id": obj.get("way_id"),
            "weighted_grade": obj.get("weighted_grade"),
            "max_upward_grade": obj.get("max_upward_grade"),
            "max_downward_grade": obj.get("max_downward_grade"),
            "mean_elevation": obj.get("mean_elevation"),
            "lane_count": obj.get("lane_count"),
            "cycle_lane": obj.get("cycle_lane"),
            "bicycle_network": obj.get("bicycle_network"),
            "sac_scale": obj.get("sac_scale"),
            "sidewalk": obj.get("sidewalk"),
            "density": obj.get("density"),
            "speed_limit": obj.get("speed_limit"),
            "truck_speed": obj.get("truck_speed"),
            "truck_route": obj.get("truck_route"),
            "end_node": EndNode.from_dict(obj.get("end_node")) if obj.get("end_node") is not None else None
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj




# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.4
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional, Union
from pydantic import BaseModel, Field, StrictBool, StrictFloat, StrictInt, confloat, conint

class TruckCostingOptions(BaseModel):
    """
    TruckCostingOptions
    """
    maneuver_penalty: Optional[StrictInt] = Field(5, description="A penalty (in seconds) applied when transitioning between roads (determined by name).")
    gate_cost: Optional[StrictInt] = Field(15, description="The estimated cost (in seconds) when a gate is encountered.")
    gate_penalty: Optional[StrictInt] = Field(300, description="A penalty (in seconds) applied to the route cost when a gate is encountered. This penalty can be used to reduce the likelihood of suggesting a route with gates unless absolutely necessary.")
    country_crossing_cost: Optional[StrictInt] = Field(600, description="The estimated cost (in seconds) when encountering an international border.")
    country_crossing_penalty: Optional[StrictInt] = Field(0, description="A penalty applied to transitions to international border crossings. This penalty can be used to reduce the likelihood of suggesting a route with border crossings unless absolutely necessary.")
    service_penalty: Optional[StrictInt] = Field(None, description="A penalty applied to transitions to service roads. This penalty can be used to reduce the likelihood of suggesting a route with service roads unless absolutely necessary. The default penalty is 15 for cars, busses, motor scooters, and motorcycles; and zero for others.")
    service_factor: Optional[Union[StrictFloat, StrictInt]] = Field(1, description="A factor that multiplies the cost when service roads are encountered. The default is 1.2 for cars and busses, and 1 for trucks, motor scooters, and motorcycles.")
    use_living_streets: Optional[Union[confloat(le=1, ge=0, strict=True), conint(le=1, ge=0, strict=True)]] = Field(None, description="A measure of willingness to take living streets. Values near 0 attempt to avoid them, and values near 1 will favour them. Note that as some routes may be impossible without living streets, 0 does not guarantee avoidance of them. The default value is 0 for trucks; 0.1 for other motor vehicles; 0.5 for bicycles; and 0.6 for pedestrians.")
    use_ferry: Optional[Union[confloat(le=1, ge=0, strict=True), conint(le=1, ge=0, strict=True)]] = Field(0.5, description="A measure of willingness to take ferries. Values near 0 attempt to avoid ferries, and values near 1 will favour them. Note that as some routes may be impossible without ferries, 0 does not guarantee avoidance of them.")
    height: Optional[Union[StrictFloat, StrictInt]] = Field(4.11, description="The height of the truck (in meters).")
    width: Optional[Union[StrictFloat, StrictInt]] = Field(2.6, description="The width of the truck (in meters).")
    toll_booth_cost: Optional[StrictInt] = Field(15, description="The estimated cost (in seconds) when a toll booth is encountered.")
    toll_booth_penalty: Optional[StrictInt] = Field(0, description="A penalty (in seconds) applied to the route cost when a toll booth is encountered. This penalty can be used to reduce the likelihood of suggesting a route with toll booths unless absolutely necessary.")
    ferry_cost: Optional[StrictInt] = Field(300, description="The estimated cost (in seconds) when a ferry is encountered.")
    use_highways: Optional[Union[confloat(le=1, ge=0, strict=True), conint(le=1, ge=0, strict=True)]] = Field(0.5, description="A measure of willingness to take highways. Values near 0 attempt to avoid highways, and values near 1 will favour them. Note that as some routes may be impossible without highways, 0 does not guarantee avoidance of them.")
    use_tolls: Optional[Union[confloat(le=1, ge=0, strict=True), conint(le=1, ge=0, strict=True)]] = Field(0.5, description="A measure of willingness to take toll roads. Values near 0 attempt to avoid tolls, and values near 1 will favour them. Note that as some routes may be impossible without tolls, 0 does not guarantee avoidance of them.")
    use_tracks: Optional[Union[confloat(le=1, ge=0, strict=True), conint(le=1, ge=0, strict=True)]] = Field(None, description="A measure of willingness to take track roads. Values near 0 attempt to avoid them, and values near 1 will favour them. Note that as some routes may be impossible without track roads, 0 does not guarantee avoidance of them. The default value is 0 for automobiles, busses, and trucks; and 0.5 for all other costing modes.")
    top_speed: Optional[conint(strict=True, le=252, ge=10)] = Field(140, description="The top speed (in kph) that the vehicle is capable of travelling.")
    shortest: Optional[StrictBool] = Field(False, description="If true changes the cost metric to be quasi-shortest (pure distance-based) costing. This will disable ALL other costing factors.")
    ignore_closures: Optional[StrictBool] = Field(False, description="If true, ignores all known closures. This option cannot be set if `location.search_filter.exclude_closures` is also specified.")
    include_hov2: Optional[StrictBool] = Field(False, description="If true, indicates the desire to include HOV roads with a 2-occupant requirement in the route when advantageous.")
    include_hov3: Optional[StrictBool] = Field(False, description="If true, indicates the desire to include HOV roads with a 3-occupant requirement in the route when advantageous.")
    include_hot: Optional[StrictBool] = Field(False, description="If true, indicates the desire to include toll roads which require the driver to pay a toll if the occupant requirement isn't met")
    length: Optional[Union[StrictFloat, StrictInt]] = Field(21.64, description="The length of the truck (in meters).")
    weight: Optional[Union[StrictFloat, StrictInt]] = Field(21.77, description="The weight of the truck (in tonnes).")
    axle_load: Optional[Union[StrictFloat, StrictInt]] = Field(9.07, description="The axle load of the truck (in tonnes).")
    hazmat: Optional[StrictBool] = Field(False, description="Whether or not the truck is carrying hazardous materials.")
    additional_properties: Dict[str, Any] = {}
    __properties = ["maneuver_penalty", "gate_cost", "gate_penalty", "country_crossing_cost", "country_crossing_penalty", "service_penalty", "service_factor", "use_living_streets", "use_ferry", "height", "width", "toll_booth_cost", "toll_booth_penalty", "ferry_cost", "use_highways", "use_tolls", "use_tracks", "top_speed", "shortest", "ignore_closures", "include_hov2", "include_hov3", "include_hot", "length", "weight", "axle_load", "hazmat"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> TruckCostingOptions:
        """Create an instance of TruckCostingOptions from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> TruckCostingOptions:
        """Create an instance of TruckCostingOptions from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return TruckCostingOptions.parse_obj(obj)

        _obj = TruckCostingOptions.parse_obj({
            "maneuver_penalty": obj.get("maneuver_penalty") if obj.get("maneuver_penalty") is not None else 5,
            "gate_cost": obj.get("gate_cost") if obj.get("gate_cost") is not None else 15,
            "gate_penalty": obj.get("gate_penalty") if obj.get("gate_penalty") is not None else 300,
            "country_crossing_cost": obj.get("country_crossing_cost") if obj.get("country_crossing_cost") is not None else 600,
            "country_crossing_penalty": obj.get("country_crossing_penalty") if obj.get("country_crossing_penalty") is not None else 0,
            "service_penalty": obj.get("service_penalty"),
            "service_factor": obj.get("service_factor") if obj.get("service_factor") is not None else 1,
            "use_living_streets": obj.get("use_living_streets"),
            "use_ferry": obj.get("use_ferry") if obj.get("use_ferry") is not None else 0.5,
            "height": obj.get("height") if obj.get("height") is not None else 4.11,
            "width": obj.get("width") if obj.get("width") is not None else 2.6,
            "toll_booth_cost": obj.get("toll_booth_cost") if obj.get("toll_booth_cost") is not None else 15,
            "toll_booth_penalty": obj.get("toll_booth_penalty") if obj.get("toll_booth_penalty") is not None else 0,
            "ferry_cost": obj.get("ferry_cost") if obj.get("ferry_cost") is not None else 300,
            "use_highways": obj.get("use_highways") if obj.get("use_highways") is not None else 0.5,
            "use_tolls": obj.get("use_tolls") if obj.get("use_tolls") is not None else 0.5,
            "use_tracks": obj.get("use_tracks"),
            "top_speed": obj.get("top_speed") if obj.get("top_speed") is not None else 140,
            "shortest": obj.get("shortest") if obj.get("shortest") is not None else False,
            "ignore_closures": obj.get("ignore_closures") if obj.get("ignore_closures") is not None else False,
            "include_hov2": obj.get("include_hov2") if obj.get("include_hov2") is not None else False,
            "include_hov3": obj.get("include_hov3") if obj.get("include_hov3") is not None else False,
            "include_hot": obj.get("include_hot") if obj.get("include_hot") is not None else False,
            "length": obj.get("length") if obj.get("length") is not None else 21.64,
            "weight": obj.get("weight") if obj.get("weight") is not None else 21.77,
            "axle_load": obj.get("axle_load") if obj.get("axle_load") is not None else 9.07,
            "hazmat": obj.get("hazmat") if obj.get("hazmat") is not None else False
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj

